@@ -17,23 +17,39 @@
 
 import string
 
-import cGrid
+import cPalabra
 import constants
-import cView
+
+def decompose_word(word, x, y, direction):
+    """Decompose the word starting at (x, y) in the given direction into tuples."""
+    if direction == "across":
+        return [(x + i, y, word[i]) for i in xrange(len(word))]
+    elif direction == "down":
+        return [(x, y + j, word[j]) for j in xrange(len(word))]
+
+def determine_scrabble_score(grid):
+    """Compute the sum of each character's score in Scrabble."""
+    # http://en.wikipedia.org/wiki/Scrabble_letter_distributions#English
+    scores = {'A': 1, 'B': 3, 'C': 3, 'D': 2, 'E': 1, 'F': 4, 'G': 2
+        , 'H': 4, 'I': 1, 'J': 8, 'K': 5, 'L': 1, 'M': 3, 'N': 1
+        , 'O': 1, 'P': 3, 'Q': 10, 'R': 1, 'S': 1, 'T': 1, 'U': 1
+        , 'V': 4, 'W': 4, 'X': 8, 'Y': 4, 'Z': 10, '': 0}
+    chars = [grid.get_char(x, y) for x, y in grid.cells()]
+    return sum([scores[c] for c in chars])
 
 class Grid:
-    def __init__(self, width, height, initialize=True):
+    def __init__(self, width, height, initialize=True, number_mode=constants.NUMBERING_AUTO):
         """Construct a grid with the given dimensions."""
-        self.initialize(width, height, initialize)
+        self.initialize(width, height, initialize, number_mode)
         
-    def initialize(self, width, height, initialize=True):
+    def initialize(self, width, height, initialize=True, number_mode=constants.NUMBERING_AUTO):
         """Reset the grid to the given dimensions with all empty cells."""
         self.width = width
         self.height = height
         self.lines = None
         self.data = [[self._default_cell() for x in xrange(width)] for y in xrange(height)]
-        # TODO modify when arbitrary number schemes are implemented
-        if initialize:
+        self.number_mode = number_mode
+        if initialize and number_mode == constants.NUMBERING_AUTO:
             self.assign_numbers()
         
     def _default_cell(self):
@@ -44,6 +60,7 @@ class Grid:
             , "clues": {}
             , "number": 0
             , "void": False
+            , "rebus": None
         }
         
     def __eq__(self, other):
@@ -59,25 +76,44 @@ class Grid:
     def __ne__(self, other):
         return not self.__eq__(other)
         
+    def __str__(self):
+        s = []
+        for y in xrange(self.height):
+            for x in xrange(self.width):
+                if self.data[y][x]["block"]:
+                    s.append('.')
+                elif self.data[y][x]["void"]:
+                    s.append('~')
+                else:
+                    c = self.data[y][x]["char"]
+                    s.append(c if c else ' ')
+            s.append('\n')
+        return ''.join(s)
+        
     def assign_numbers(self):
         """Assign word numbers to cells as they are commonly numbered."""
-        cGrid.assign_numbers(self)
+        cPalabra.assign_numbers(self)
             
     def is_start_word(self, x, y, direction=None):
         """Return True when a word begins in the cell (x, y)."""
-        is_available = self.is_available
-        data = self.data
-        if not is_available(x, y):
+        width, height, data = self.width, self.height, self.data
+        if not (0 <= x < width and 0 <= y < height):
+            return False
+        if data[y][x]["block"] or data[y][x]["void"]:
             return False
         for d in ([direction] if direction else ["across", "down"]):
             if d == "across":
                 bdx, bdy, adx, ady, bar_side = -1, 0, 1, 0, "left"
             elif d == "down":
                 bdx, bdy, adx, ady, bar_side = 0, -1, 0, 1, "top"
-            before = not is_available(x + bdx, y + bdy) or data[y][x]["bar"][bar_side]
+            is_available_xbdx_ybdy = ((0 <= x + bdx < width and 0 <= y + bdy < height)
+                and not data[y + bdy][x + bdx]["block"] and not data[y + bdy][x + bdx]["void"])
+            before = not is_available_xbdx_ybdy or data[y][x]["bar"][bar_side]
             if not before:
                 continue
-            after = is_available(x + adx, y + ady) and not data[y + ady][x + adx]["bar"][bar_side]
+            is_available_xadx_yady = ((0 <= x + adx < width and 0 <= y + ady < height)
+                and not data[y + ady][x + adx]["block"] and not data[y + ady][x + adx]["void"])
+            after = is_available_xadx_yady and not data[y + ady][x + adx]["bar"][bar_side]
             if after:
                 return True
         return False
@@ -265,8 +301,9 @@ class Grid:
                 elif counts[x, y] == 2:
                     status["checked_count"] += 1
                 status["clue_count"] += len(self.data[y][x]["clues"])
-            status["open_count"] = self.count_open_squares()
+            status["open_count"] = len(self.compute_open_squares())
             status["connected"] = self.is_connected()
+            status["complete_count"] = self.count_complete()
         return status
         
     def determine_word_counts(self):
@@ -295,16 +332,21 @@ class Grid:
         
     def words_by_direction(self, direction):
         """Iterate over the words in the grid by direction."""
-        for y in xrange(self.height):
-            for x in xrange(self.width):
-                if self.is_start_word(x, y, direction):
-                    yield self.data[y][x]["number"], x, y
+        height = self.height
+        width = self.width
+        is_start_word = self.is_start_word
+        data = self.data
+        for y in xrange(height):
+            for x in xrange(width):
+                if is_start_word(x, y, direction):
+                    yield data[y][x]["number"], x, y
                     
     def clues(self, direction):
         """Iterate over the clues of the grid by direction."""
+        data = self.data
         for n, x, y in self.words_by_direction(direction):
             try:
-                yield n, x, y, self.data[y][x]["clues"][direction]
+                yield n, x, y, data[y][x]["clues"][direction]
             except KeyError:
                 yield n, x, y, {}
     
@@ -319,18 +361,19 @@ class Grid:
         include_dir: If True, include the direction in the output.
         allow_duplicates must be True for this to work.
         """
+        data = self.data
+        is_start_word = self.is_start_word
         for x, y in self.cells():
-            n = self.cell(x, y)["number"]
+            n = data[y][x]["number"]
             if allow_duplicates:
                 for d in ["across", "down"]:
-                    if self.is_start_word(x, y, d):
+                    if is_start_word(x, y, d):
                         if include_dir:
                             yield n, x, y, d
                         else:
                             yield n, x, y
-            else:
-                if self.is_start_word(x, y):
-                    yield n, x, y
+            elif is_start_word(x, y):
+                yield n, x, y
     
     def entries(self):
         """Return all entries in alphabetical order."""
@@ -361,25 +404,18 @@ class Grid:
     def gather_word(self, x, y, direction, empty_char=constants.MISSING_CHAR):
         """Return the word starting at (x, y) in the given direction."""
         word = ""
+        data = self.data
         for p, q in self.in_direction(x, y, direction):
-            c = self.data[q][p]["char"]
+            c = data[q][p]["char"]
             word += (empty_char if c == "" else c)
         return word
-        
-    @staticmethod
-    def decompose_word(word, x, y, direction):
-        """Decompose the word starting at (x, y) in the given direction into tuples."""
-        if direction == "across":
-            return [(x + i, y, word[i]) for i in xrange(len(word))]
-        elif direction == "down":
-            return [(x, y + j, word[j]) for j in xrange(len(word))]
             
     def gather_constraints(self, x, y, direction):
         """Create a list of all chars by position of the specified word."""
         word = self.gather_word(x, y, direction, constants.MISSING_CHAR)
         return [(i, c.lower()) for i, c in enumerate(word) if c != constants.MISSING_CHAR]
     
-    def gather_all_constraints(self, x, y, direction):
+    def gather_all_constraints(self, x, y, direction, g_cs=None, g_lengths=None):
         """
         Gather constraints of all intersecting words of the word at (x, y).
         
@@ -399,20 +435,29 @@ class Grid:
         sx, sy = get_start_word(x, y, direction)
         for s, t in self.in_direction(sx, sy, direction):
             p, q = get_start_word(s, t, other)
-            length = word_length(p, q, other)
+            if g_lengths is not None and (p, q, other) in g_lengths:
+                length = g_lengths[p, q, other]
+            else:
+                length = word_length(p, q, other)
             if other == "across":
                 index = x - p
             elif other == "down":
                 index = y - q
-            constraints = gather_constraints(p, q, other)
-            result.append((index, length, constraints))
+            if g_cs is not None and (p, q, other) in g_cs:
+                cs = g_cs[p, q, other]
+            else:
+                cs = gather_constraints(p, q, other)
+            result.append((index, length, cs))
         return result
                 
     def gather_words(self, direction=None):
         """Iterate over the word data in the given direction."""
+        words_by_direction = self.words_by_direction
+        data = self.data
+        gather_word = self.gather_word
         for d in ([direction] if direction else ["across", "down"]):
-            for n, x, y in self.words_by_direction(d):
-                clues = self.data[y][x]["clues"]
+            for n, x, y in words_by_direction(d):
+                clues = data[y][x]["clues"]
                 try:
                     clue = clues[d]["text"]
                 except KeyError:
@@ -421,8 +466,7 @@ class Grid:
                     explanation = clues[d]["explanation"]
                 except KeyError:
                     explanation = ""
-                    
-                word = self.gather_word(x, y, d)
+                word = gather_word(x, y, d)
                 yield n, x, y, d, word, clue, explanation
         
     def word_length(self, x, y, direction):
@@ -500,15 +544,15 @@ class Grid:
             if 0 <= x + dx < width and 0 <= y + dy < height:
                 yield x + dx, y + dy
         
-    def count_open_squares(self):
+    def compute_open_squares(self):
         """
-        Return the number of open squares.
+        Return the open squares.
         
         A square is open if it does not touch a block, including diagonally.
         """
         data = self.data
         neighbors = self.neighbors
-        count = 0
+        cells = []
         for x, y in self.cells():
             if data[y][x]["block"] or data[y][x]["void"]:
                 continue
@@ -516,8 +560,8 @@ class Grid:
                 if data[q][p]["block"]:
                     break
             else:
-                count += 1
-        return count
+                cells.append((x, y))
+        return cells
         
     def is_connected(self):
         """
@@ -550,6 +594,12 @@ class Grid:
                     total += 1
         return total
         
+    def get_cell_of_slot(self, slot, target):
+        if target == "start":
+            return self.get_start_word(*slot)
+        elif target == "end":
+            return self.get_end_word(*slot)
+        
     def count_words(self):
         """Return the number of words in the grid."""
         count = 0
@@ -580,15 +630,22 @@ class Grid:
                     count += 1
         return count
         
-    def count_chars(self, include_blanks=True):
+    def count_chars(self, include_blanks=True, _return_at_one=False):
         """Return the number of chars in the grid."""
         count = 0
         for y in xrange(self.height):
             for x in xrange(self.width):
-                if (not self.data[y][x]["block"] and not self.data[y][x]["void"]):
-                    if include_blanks or self.data[y][x]["char"] != '':
-                        count += 1
+                if (self.data[y][x]["block"] or self.data[y][x]["void"]):
+                    continue
+                if include_blanks or self.data[y][x]["char"] != '':
+                    count += 1
+                    if _return_at_one:
+                        return count
         return count
+        
+    def has_chars(self):
+        """Return True iff the grid has at least one character."""
+        return self.count_chars(include_blanks=False, _return_at_one=True) == 1
             
     def mean_word_length(self):
         """Return the mean length of the words in the grid."""
@@ -597,6 +654,80 @@ class Grid:
             return 0
         char_counts = [self.get_check_count(x, y) for x, y in self.availables()]
         return float(sum(char_counts)) / float(word_count)
+        
+    def generate_diagonals(self, direction=0):
+        """
+        Generate all sequences of diagonal character cells of length 2+.
+        
+        Direction: 0 = north-east, 1 = south-east.
+        """
+        data = self.data
+        width = self.width
+        height = self.height
+        if direction == 0:
+            cs1 = [(0, y) for y in xrange(1, height)]
+            cs2 = [(x, height - 1) for x in xrange(1, width - 1)]
+            dx, dy = 1, -1
+        elif direction == 1:
+            cs1 = [(0, y) for y in xrange(height)]
+            cs2 = [(x, 0) for x in xrange(1, width - 1)]
+            dx, dy = 1, 1
+        seqs = []
+        for cx, cy in (cs1 + cs2):
+            cells = []
+            while (0 <= cx < width and 0 <= cy < height):
+                if not data[cy][cx]["block"] and not data[cy][cx]["void"]:
+                    cells.append((cx, cy))
+                else:
+                    cells.append(None)
+                cx += dx
+                cy += dy
+            seqs.append(cells)
+        result = []
+        for s in seqs:
+            sr = []
+            t = []
+            for e in s:
+                if e is None:
+                    sr.append(t)
+                    t = []
+                else:
+                    t.append(e)
+            if t:
+                sr.append(t)
+            result.extend([i for i in sr if len(i) >= 2])
+        return result
+        
+    def generate_all_slots(self):
+        """
+        Generate all sequences of cells of length 2+ in the directions across,
+        across (reverse), down, down (reverse), north-east, south-east,
+        south-west and north-west.
+        """
+        data = self.data
+        gather_word = self.gather_word
+        slots = []
+        for n, x, y, d in self.words(allow_duplicates=True, include_dir=True):
+            l1 = decompose_word(gather_word(x, y, d), x, y, d)
+            l2 = decompose_word(gather_word(x, y, d), x, y, d)
+            l2.reverse()
+            slots.append((d, l1))
+            slots.append((d + "r", l2))
+        opp = {"ne": "sw", "se": "nw"}
+        for i, d in [(0, "ne"), (1, "se")]:
+            seqs = self.generate_diagonals(i)
+            for s in seqs:
+                t1, t2 = [], []
+                for x, y in s:
+                    c = data[y][x]["char"]
+                    if c == '':
+                        c = constants.MISSING_CHAR
+                    t1.append((x, y, c))
+                    t2.append((x, y, c))
+                t2.reverse()
+                slots.append((d, t1))
+                slots.append((opp[d], t2))
+        return slots
             
     def resize(self, width, height, make_dirty=True):
         """
@@ -625,6 +756,16 @@ class Grid:
         # invalidate lines
         self.lines = None
         
+    def count_complete(self):
+        r = {"across": 0, "down": 0}
+        for n, x, y, d in self.words(allow_duplicates=True, include_dir=True):
+            for p, q in self.in_direction(x, y, d):
+                if self.data[q][p]["char"] == '':
+                    break
+            else:
+                r[d] += 1
+        return r
+        
     def _clear_clues(self, dirty_cells):
         """Remove the clues of the words that contain a dirty cell."""
         for x, y, direction in dirty_cells:
@@ -910,6 +1051,7 @@ class Grid:
     def set_block(self, x, y, status):
         self._on_cell_type_change(x, y, status)
         self.data[y][x]["block"] = status
+        self.data[y][x]["void"] = False
         
     def is_block(self, x, y):
         return self.data[y][x]["block"]
@@ -952,6 +1094,12 @@ class Grid:
         self._on_cell_type_change(x, y, status)
         self.data[y][x]["void"] = status
         
+    def has_rebus(self, x, y):
+        return self.data[y][x]["rebus"] is not None
+        
+    def set_rebus(self, x, y, short, full):
+        self.data[y][x]["rebus"] = (short, full)
+        
     def _on_cell_type_change(self, x, y, status):
         if status:
             self._clear_clues_related_to_cell(x, y)
