@@ -37,12 +37,11 @@ import webbrowser
 import action
 from action import State
 from appearance import AppearanceDialog
-import cGrid
-import cWord
+import cPalabra
 from clue import ClueTool
 import constants
-from export import ExportWindow, verify_output_options
-from editor import Editor, FillTool, WordTool
+from export import ExportWindow
+from editor import e_settings, e_tools, Editor
 from files import (
     FILETYPES,
     ParserError,
@@ -50,13 +49,19 @@ from files import (
     export_puzzle,
     read_containers,
 )
+from gui_editor import WordTool, FillTool
+from gui_prefs import PreferencesWindow
+from gui_word import (
+    AccidentalWordsDialog,
+    FindWordsDialog,
+    WordListEditor,
+)
 import grid
 from grid import Grid
 from newpuzzle import NewWindow, SizeWindow
-from pattern import PatternFileEditor, PatternEditor
+from pattern import PatternFileEditor
 import preferences
 from preferences import (
-    PreferencesWindow,
     write_config_file,
     read_config_file,
 )
@@ -64,7 +69,7 @@ from properties import PropertiesWindow
 from puzzle import Puzzle, PuzzleManager
 import transform
 import view
-from word import create_wordlists, WordListEditor
+from word import create_wordlists
 
 def create_splash():
     window = gtk.Window(gtk.WINDOW_TOPLEVEL)
@@ -83,7 +88,7 @@ def create_splash():
 class PalabraWindow(gtk.Window):
     def __init__(self):
         super(PalabraWindow, self).__init__()
-        self.set_title("Palabra")
+        self.set_title(constants.TITLE)
         self.set_size_request(800, 600)
         
         self.puzzle_toggle_items = []
@@ -91,16 +96,14 @@ class PalabraWindow(gtk.Window):
         
         self.puzzle_manager = PuzzleManager()
         
-        self.editor_settings = None
-        
         self.menubar = gtk.MenuBar()
         self.menubar.append(self.create_file_menu())
         self.menubar.append(self.create_edit_menu())
         self.menubar.append(self.create_view_menu())
         self.menubar.append(self.create_grid_menu())
-        #self.menubar.append(self.create_fill_menu())
         #self.menubar.append(self.create_word_menu())
         #self.menubar.append(self.create_clue_menu())
+        self.menubar.append(self.create_tools_menu())
         self.menubar.append(self.create_help_menu())
         
         self.toolbar = self.create_toolbar()
@@ -119,7 +122,7 @@ class PalabraWindow(gtk.Window):
         self.connect("delete-event", self.on_delete)
         self.connect("destroy", lambda widget: quit())
         
-        self.wordlists = {}
+        self.wordlists = []
         self.patterns = None
         
         self.blacklist = None
@@ -136,25 +139,34 @@ class PalabraWindow(gtk.Window):
     def to_empty_panel(self):
         for widget in self.panel.get_children():
             self.panel.remove(widget)
-        self.editor.cleanup()
+        self.drawing_area.unset_flags(gtk.CAN_FOCUS)
+        for i in self.ids:
+            self.drawing_area.disconnect(i)
     
     def to_edit_panel(self):
-        drawing_area = gtk.DrawingArea()
-        drawing_area.add_events(
+        self.drawing_area = gtk.DrawingArea()
+        self.drawing_area.set_flags(gtk.CAN_FOCUS)
+        self.drawing_area.add_events(
             gtk.gdk.BUTTON_PRESS_MASK
             | gtk.gdk.BUTTON_RELEASE_MASK
             | gtk.gdk.POINTER_MOTION_MASK
             | gtk.gdk.KEY_PRESS_MASK
             | gtk.gdk.KEY_RELEASE_MASK
+            | gtk.gdk.POINTER_MOTION_HINT_MASK
             )
         puzzle = self.puzzle_manager.current_puzzle
-        puzzle.view.refresh_visual_size(drawing_area)
-        drawing_area.queue_draw()
-        
-        self.editor = Editor(self, drawing_area)
-        self.editor.tools["clue"] = ClueTool(self.editor)
-        self.editor.tools["fill"] = FillTool(self.editor)
-        self.editor.tools["word"] = WordTool(self.editor)
+        puzzle.view.refresh_visual_size(self.drawing_area)
+        self.drawing_area.queue_draw()
+        self.editor = Editor(self)
+        self.ids = []
+        for k, e in self.editor.EVENTS.items():
+            if isinstance(e, tuple):
+                self.ids.append(self.drawing_area.connect(k, *e))
+            else:
+                self.ids.append(self.drawing_area.connect(k, e))
+        e_tools["clue"] = ClueTool(self.editor)
+        e_tools["fill"] = FillTool(self.editor)
+        e_tools["word"] = WordTool(self.editor)
         
         options_hbox = gtk.HBox(False, 0)
         options_hbox.set_border_width(12)
@@ -169,7 +181,7 @@ class PalabraWindow(gtk.Window):
         
         scrolled_window = gtk.ScrolledWindow(None, None)
         scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        scrolled_window.add_with_viewport(drawing_area)
+        scrolled_window.add_with_viewport(self.drawing_area)
         main = gtk.VBox(False, 0)
         main.pack_start(scrolled_window, True, True, 0)
         
@@ -179,20 +191,20 @@ class PalabraWindow(gtk.Window):
         tabs.set_show_border(False)
         tabs.set_property("tab-hborder", 16)
         tabs.set_property("tab-vborder", 8)
-        tool = self.editor.tools["word"].create()
+        tool = e_tools["word"].create()
         tabs.append_page(tool, gtk.Label(u"Word"))
-        tool = self.editor.tools["fill"].create()
+        #tool = e_tools["fill"].create()
         #tabs.append_page(tool, gtk.Label(u"Fill"))
-        tool = self.editor.tools["clue"].create(puzzle)
+        tool = e_tools["clue"].create(puzzle)
         tabs.append_page(tool, gtk.Label(u"Clue"))
         def on_switch_page(tabs, do_not_use, num):
             if num == 0:
-                word = self.editor.tools["word"].get_selected_word()
+                word = e_tools["word"].get_selected_word()
                 self.editor.set_overlay(word)
                 self.update_window()
             else:
                 self.editor.set_overlay(None)
-                self.editor.tools["word"].deselect()
+                e_tools["word"].deselect()
         tabs.connect("switch-page", on_switch_page)
         
         paned = gtk.HPaned()
@@ -203,15 +215,20 @@ class PalabraWindow(gtk.Window):
         self.panel.pack_start(paned, True, True, 0)
         self.panel.show_all()
         
-    def get_selection(self):
-        try:
-            return self.editor.get_selection()
-        except AttributeError:
-            return None
+    def get_selection(self, slot=False):
+        x, y, d = e_settings.selection
+        if slot:
+            puzzle = self.puzzle_manager.current_puzzle
+            sx, sy = puzzle.grid.get_start_word(x, y, d)
+            return (x, y, d, puzzle.grid.word_length(sx, sy, d))
+        return (x, y)
             
-    def set_selection(self, x, y, direction=None):
+    def set_selection(self, x, y, direction=None, selection_changed=True):
         try:
-            self.editor.set_selection(x, y, direction)
+            self.editor.set_selection(x=x
+                , y=y
+                , direction=direction
+                , selection_changed=selection_changed)
         except AttributeError:
             pass
         
@@ -308,16 +325,16 @@ class PalabraWindow(gtk.Window):
             dialog.destroy()
             
     def update_title(self, path=None):
-        title = u"Unsaved puzzle - Palabra"
+        title = u"Unsaved puzzle - " + constants.TITLE
         if path is not None:
             filename_start = path.rfind(os.sep) + 1
             filename = path[filename_start:]
-            title = ''.join([filename, u" - Palabra"])
+            title = ''.join([filename, u" - ", constants.TITLE])
         self.set_title(title)
     
     def save_puzzle(self, save_as=False):
         puzzle = self.puzzle_manager.current_puzzle
-        backup = preferences.prefs["backup_copy_before_save"]
+        backup = preferences.prefs[constants.PREF_COPY_BEFORE_SAVE]
         if save_as or self.puzzle_manager.current_puzzle.filename is None:
             title = u"Save puzzle"
             if save_as:
@@ -357,36 +374,21 @@ class PalabraWindow(gtk.Window):
         action.stack.distance_from_saved = 0
         
     def export_puzzle(self):
-        window = ExportWindow(self)
-        window.show_all()
-        
-        response = window.run()
-        if response == gtk.RESPONSE_OK:
-            window.hide()
-            
-            options = window.options
-            message = verify_output_options(options)
-            if message is not None:
-                mdialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL
-                    , gtk.MESSAGE_INFO, gtk.BUTTONS_NONE, message)
-                mdialog.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
-                mdialog.run()
-                mdialog.destroy()
-            else:
-                dialog = gtk.FileChooserDialog(u"Export location"
-                    , self
-                    , gtk.FILE_CHOOSER_ACTION_SAVE
-                    , (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL
-                    , gtk.STOCK_SAVE, gtk.RESPONSE_OK))
-                dialog.set_do_overwrite_confirmation(True)
-                
-                dialog.show_all()
-                response = dialog.run()
-                if response == gtk.RESPONSE_OK:
-                    export_puzzle(self.puzzle_manager.current_puzzle
-                        , dialog.get_filename(), options)
-                dialog.destroy()
-        window.destroy()
+        w = ExportWindow(self)
+        w.show_all() 
+        if w.run() == gtk.RESPONSE_OK:
+            w.hide()
+            d = gtk.FileChooserDialog(u"Export location", self
+                , gtk.FILE_CHOOSER_ACTION_SAVE
+                , (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL
+                , gtk.STOCK_SAVE, gtk.RESPONSE_OK))
+            d.set_do_overwrite_confirmation(True)
+            d.show_all() 
+            if d.run() == gtk.RESPONSE_OK:
+                export_puzzle(self.puzzle_manager.current_puzzle
+                    , d.get_filename(), w.options)
+            d.destroy()
+        w.destroy()
     
     def load_puzzle(self):
         action.stack.push(State(self.puzzle_manager.current_puzzle.grid), initial=True)
@@ -692,88 +694,51 @@ class PalabraWindow(gtk.Window):
         
     def create_edit_symmetry_menu(self):
         menu = gtk.Menu()
-        
-        def set_symmetry(options):
+        def set_symmetry(item, options):
             try:
-                self.editor.settings["symmetries"] = options
+                e_settings.settings["symmetries"] = options
             except AttributeError:
                 pass
-        
-        activate = lambda item: set_symmetry([])
-        select = lambda item: self.update_status(constants.STATUS_MENU
-            , u"Use no symmetry rules when modifying the grid")
-        deselect = lambda item: self.pop_status(constants.STATUS_MENU)
-        item = gtk.RadioMenuItem(None, u"_No symmetry", True)
-        item.connect("activate", activate)
-        item.connect("select", select)
-        item.connect("deselect", deselect)
-        menu.append(item)
-        
-        menu.append(gtk.SeparatorMenuItem())
-        
-        activate = lambda item: set_symmetry(["horizontal"])
-        select = lambda item: self.update_status(constants.STATUS_MENU
-            , u"Use a horizontal symmetry axis when modifying the grid")
-        deselect = lambda item: self.pop_status(constants.STATUS_MENU)
-        item = gtk.RadioMenuItem(item, u"_Horizontal symmetry axis", True)
-        item.connect("activate", activate)
-        item.connect("select", select)
-        item.connect("deselect", deselect)
-        menu.append(item)
-        
-        activate = lambda item: set_symmetry(["vertical"])
-        select = lambda item: self.update_status(constants.STATUS_MENU
-            , u"Use a vertical symmetry axis when modifying the grid")
-        deselect = lambda item: self.pop_status(constants.STATUS_MENU)
-        item = gtk.RadioMenuItem(item, u"_Vertical symmetry axis", True)
-        item.connect("activate", activate)
-        item.connect("select", select)
-        item.connect("deselect", deselect)
-        menu.append(item)
-        
-        activate = lambda item: set_symmetry(["horizontal", "vertical"])
-        select = lambda item: self.update_status(constants.STATUS_MENU
-            , u"Use horizontal and vertical symmetry axes when modifying the grid")
-        deselect = lambda item: self.pop_status(constants.STATUS_MENU)
-        item = gtk.RadioMenuItem(item, u"_Horizontal and vertical axes", True)
-        item.connect("activate", activate)
-        item.connect("select", select)
-        item.connect("deselect", deselect)
-        menu.append(item)
-        
-        activate = lambda item: set_symmetry(["diagonals"])
-        select = lambda item: self.update_status(constants.STATUS_MENU
-            , u"Use diagonal symmetry axes when modifying the grid")
-        deselect = lambda item: self.pop_status(constants.STATUS_MENU)
-        item = gtk.RadioMenuItem(item, u"_Diagonal symmetry axes", True)
-        item.connect("activate", activate)
-        item.connect("select", select)
-        item.connect("deselect", deselect)
-        menu.append(item)
-        
-        menu.append(gtk.SeparatorMenuItem())
-
-        activate = lambda item: set_symmetry(["90_degree"])
-        select = lambda item: self.update_status(constants.STATUS_MENU
-            , u"Use 90 degree rotational symmetry when modifying the grid")
-        deselect = lambda item: self.pop_status(constants.STATUS_MENU)
-        item = gtk.RadioMenuItem(item, u"_90 degree rotational symmetry", True)
-        item.connect("activate", activate)
-        item.connect("select", select)
-        item.connect("deselect", deselect)
-        menu.append(item)
-        
-        activate = lambda item: set_symmetry(["180_degree"])
-        select = lambda item: self.update_status(constants.STATUS_MENU
-            , u"Use 180 degree rotational symmetry when modifying the grid")
-        deselect = lambda item: self.pop_status(constants.STATUS_MENU)
-        item = gtk.RadioMenuItem(item, u"_180 degree rotational symmetry", True)
-        item.connect("activate", activate)
-        item.connect("select", select)
-        item.connect("deselect", deselect)
-        item.set_active(True)
-        menu.append(item)
-        
+        def create_symmetry_option(symmetries, txt_select, txt_item, prev, active):
+            select = lambda i: self.update_status(constants.STATUS_MENU, txt_select)
+            deselect = lambda i: self.pop_status(constants.STATUS_MENU)
+            item = gtk.RadioMenuItem(prev, txt_item, True)
+            item.connect("activate", set_symmetry, symmetries)
+            item.connect("select", select)
+            item.connect("deselect", deselect)
+            if active:
+                item.set_active(True)
+            menu.append(item)
+            return item
+        item = None
+        options = [
+            ([], u"Use no symmetry rules when modifying the grid", u"_No symmetry", False)
+            , (None, None, None, None)
+            , ([constants.SYM_HORIZONTAL]
+                , u"Use a horizontal symmetry axis when modifying the grid"
+                , u"_Horizontal symmetry axis", False)
+            , ([constants.SYM_VERTICAL]
+                , u"Use a vertical symmetry axis when modifying the grid"
+                , u"_Vertical symmetry axis", False)
+            , ([constants.SYM_HORIZONTAL, constants.SYM_VERTICAL]
+                , u"Use horizontal and vertical symmetry axes when modifying the grid"
+                , u"_Horizontal and vertical axes", False)
+            , ([constants.SYM_DIAGONALS]
+                , u"Use diagonal symmetry axes when modifying the grid"
+                , u"_Diagonal symmetry axes", False)
+            , (None, None, None, None)
+            , ([constants.SYM_90]
+                , u"Use 90 degree rotational symmetry when modifying the grid"
+                , u"_90 degree rotational symmetry", False)
+            , ([constants.SYM_180]
+                , u"Use 180 degree rotational symmetry when modifying the grid"
+                , u"_180 degree rotational symmetry", True)
+        ]
+        for o_s, o_sel, o_item, o_active in options:
+            if o_s is None:
+                menu.append(gtk.SeparatorMenuItem())
+            else:
+                item = create_symmetry_option(o_s, o_sel, o_item, item, o_active)
         symmetry_menu = gtk.MenuItem(u"_Symmetry", True)
         symmetry_menu.set_submenu(menu)
         return symmetry_menu
@@ -785,8 +750,10 @@ class PalabraWindow(gtk.Window):
             self.transform_grid(transform, x=sel_x, y=sel_y)
     
     def transform_grid(self, transform, **args):
+        pre_slot = self.get_selection(slot=True)
         puzzle = self.puzzle_manager.current_puzzle
         transform(puzzle, **args)
+        selection_changed = pre_slot != self.get_selection(slot=True)
         action.stack.push(State(self.puzzle_manager.current_puzzle.grid))
         # TODO catch all, should not be needed
         if puzzle.grid.lines:
@@ -799,7 +766,7 @@ class PalabraWindow(gtk.Window):
             t = transform.type
         except AttributeError:
             t = constants.TRANSFORM_STRUCTURE
-        self.update_window(True, transform=t)        
+        self.update_window(True, transform=t, selection_changed=selection_changed)
         
     def transform_clues(self, transform, **args):
         puzzle = self.puzzle_manager.current_puzzle
@@ -816,17 +783,25 @@ class PalabraWindow(gtk.Window):
         self.undo_tool_item.set_sensitive(has_undo)
         self.redo_tool_item.set_sensitive(has_redo)
         
-    def update_window(self, content_changed=False, transform=constants.TRANSFORM_STRUCTURE):
+    def update_window(self
+        , content_changed=False
+        , transform=constants.TRANSFORM_STRUCTURE
+        , selection_changed=True):
         puzzle = self.puzzle_manager.current_puzzle
         if puzzle is None:
-            self.set_title(u"Palabra")
+            self.set_title(constants.TITLE)
             self.pop_status(constants.STATUS_GRID)
         else:
+            selection = self.get_selection()
             if content_changed and transform >= constants.TRANSFORM_STRUCTURE:
                 status = puzzle.grid.determine_status(False)
                 message = self.determine_status_message(status)
                 self.update_status(constants.STATUS_GRID, message)
-            selection = self.get_selection()
+                # if grid structure changes, set selection again
+                # (it may have changed)
+                if selection is not None:
+                    sel_x, sel_y = selection
+                    self.set_selection(x=sel_x, y=sel_y, selection_changed=selection_changed)
             if selection is not None:
                 sel_x, sel_y = selection
                 valid = puzzle.grid.is_valid(sel_x, sel_y)
@@ -910,6 +885,14 @@ class PalabraWindow(gtk.Window):
                 self.panel.queue_draw()
             except AttributeError:
                 pass
+                
+        def toggle_warning(predicate, status):
+            e_settings.warnings[predicate] = status
+            try:
+                self.editor.force_redraw = True
+                self.panel.queue_draw()
+            except AttributeError:
+                pass
         
         activate = lambda item: toggle_predicate("show_numbers", item.active)
         select = lambda item: self.update_status(constants.STATUS_MENU
@@ -925,7 +908,7 @@ class PalabraWindow(gtk.Window):
         
         menu.append(gtk.SeparatorMenuItem())
         
-        activate = lambda item: toggle_predicate("warn_unchecked_cells", item.active)
+        activate = lambda item: toggle_warning(constants.WARN_UNCHECKED, item.active)
         select = lambda item: self.update_status(constants.STATUS_MENU
             , u"Warn visually when cells that belong to one word exist in the grid")
         deselect = lambda item: self.pop_status(constants.STATUS_MENU)
@@ -935,9 +918,9 @@ class PalabraWindow(gtk.Window):
         item.connect("deselect", deselect)
         menu.append(item)
         item.set_active(True)
-        toggle_predicate("warn_unchecked_cells", True)
+        toggle_warning(constants.WARN_UNCHECKED, True)
         
-        activate = lambda item: toggle_predicate("warn_consecutive_unchecked", item.active)
+        activate = lambda item: toggle_warning(constants.WARN_CONSECUTIVE, item.active)
         select = lambda item: self.update_status(constants.STATUS_MENU
             , u"Warn visually when consecutive unchecked cells exist in the grid")
         deselect = lambda item: self.pop_status(constants.STATUS_MENU)
@@ -947,9 +930,9 @@ class PalabraWindow(gtk.Window):
         item.connect("deselect", deselect)
         menu.append(item)
         item.set_active(True)
-        toggle_predicate("warn_consecutive_unchecked", True)
+        toggle_warning(constants.WARN_CONSECUTIVE, True)
            
-        activate = lambda item: toggle_predicate("warn_two_letter_words", item.active)
+        activate = lambda item: toggle_warning(constants.WARN_TWO_LETTER, item.active)
         select = lambda item: self.update_status(constants.STATUS_MENU
             , u"Warn visually when two-letter words exist in the grid")
         deselect = lambda item: self.pop_status(constants.STATUS_MENU)
@@ -959,7 +942,7 @@ class PalabraWindow(gtk.Window):
         item.connect("deselect", deselect)
         menu.append(item)
         item.set_active(True)
-        toggle_predicate("warn_two_letter_words", True)
+        toggle_warning(constants.WARN_TWO_LETTER, True)
         
         if False: # TODO until ready
             activate = lambda item: toggle_predicate("warn_blacklist", item.active)
@@ -983,7 +966,9 @@ class PalabraWindow(gtk.Window):
         editor = AppearanceDialog(self, puzzle.view.properties)
         editor.show_all()
         if editor.run() == gtk.RESPONSE_OK:
-            puzzle.view.properties.apply_appearance(editor.gather_appearance())
+            app = editor.gather_appearance()
+            for key, value in app.items():
+                puzzle.view.properties[key] = value
             try:
                 self.editor.force_redraw = True
                 self.editor.refresh_visual_size()
@@ -1003,11 +988,11 @@ class PalabraWindow(gtk.Window):
     def create_grid_menu(self):
         menu = gtk.Menu()
         
-        #menu.append(self._create_menu_item(
-        #    lambda item: self.edit_appearance()
-        #    , u"Edit the appearance of the puzzle"
-        #    , title=u"Edit _appearance..."
-        #    , is_puzzle_sensitive=True))
+        menu.append(self._create_menu_item(
+            lambda item: self.edit_appearance()
+            , u"Edit the appearance of the puzzle"
+            , title=u"Edit _appearance..."
+            , is_puzzle_sensitive=True))
             
         menu.append(gtk.SeparatorMenuItem())
         
@@ -1168,34 +1153,42 @@ class PalabraWindow(gtk.Window):
         clear_menu.set_submenu(menu)
         return clear_menu
         
-    def on_fill_grid(self):
-        # TODO use all wordlists
-        words = None
-        for path, item in self.wordlists.items():
-            wordlist = item["list"]
-            if wordlist is not None:
-                words = wordlist.words
-        grid = self.puzzle_manager.current_puzzle.grid
-        meta = [(x, y, 0 if d == 'across' else 1
-            , grid.word_length(x, y, d)
-            , grid.gather_constraints(x, y, d))
-            for n, x, y, d in grid.words(True, True)]
-        result = cGrid.fill(grid, words, meta)
-        if len(result) > 0:
-            self.transform_grid(transform.modify_chars, chars=result[0])
-        
-    def create_fill_menu(self):
+    def create_tools_menu(self):
         menu = gtk.Menu()
         
-        menu.append(self._create_menu_item(
-            lambda item: self.on_fill_grid()
-            , u"Fill the grid with words from the available word lists"
-            , title="_Fill grid"
-            , is_puzzle_sensitive=True))
+        def activate(item):
+            w = FindWordsDialog(self)
+            w.show_all()
+            w.run()
+            w.destroy()
+        select = lambda item: self.update_status(constants.STATUS_MENU
+            , u"Find words in wordlists according to a pattern")
+        deselect = lambda item: self.pop_status(constants.STATUS_MENU)
+        item = gtk.MenuItem(u"_Find words...", True)
+        item.connect("activate", activate, )
+        item.connect("select", select)
+        item.connect("deselect", deselect)
+        menu.append(item)
         
-        fill_menu = gtk.MenuItem(u"_Fill", True)
-        fill_menu.set_submenu(menu)
-        return fill_menu
+        def activate(item):
+            w = AccidentalWordsDialog(self, self.puzzle_manager.current_puzzle)
+            w.show_all()
+            w.run()
+            w.destroy()
+        select = lambda item: self.update_status(constants.STATUS_MENU
+            , u"View words that may have accidentally appeared in the grid")
+        deselect = lambda item: self.pop_status(constants.STATUS_MENU)
+        item = gtk.MenuItem(u"View _accidental words...", True)
+        item.connect("activate", activate)
+        item.connect("select", select)
+        item.connect("deselect", deselect)
+        menu.append(item)
+        item.set_sensitive(False)
+        self.puzzle_toggle_items += [item]
+        
+        tool_menu = gtk.MenuItem(u"_Tools", True)
+        tool_menu.set_submenu(menu)
+        return tool_menu
     
     def create_word_menu(self):
         menu = gtk.Menu()
@@ -1248,8 +1241,8 @@ class PalabraWindow(gtk.Window):
         
     def on_help_about_activate(self, widget, data=None):
         dialog = gtk.AboutDialog()
-        dialog.set_title(u"About Palabra")
-        dialog.set_program_name(u"Palabra")
+        dialog.set_title(u"About " + constants.TITLE)
+        dialog.set_program_name(constants.TITLE)
         dialog.set_comments(u"Crossword creation software")
         dialog.set_version(constants.VERSION)
         dialog.set_authors([u"Simeon Visser <simeonvisser@gmail.com>"])
@@ -1270,7 +1263,7 @@ You should have received a copy of the GNU General Public License along with thi
 
 def quit():
     gtk.main_quit()
-    cWord.postprocess()
+    cPalabra.postprocess()
     write_config_file()
     
 def main(argv=None):
@@ -1284,13 +1277,14 @@ def main(argv=None):
         print "Reading configuration file..."
         read_config_file()
         print "Loading wordlists..."
+        cPalabra.preprocess_all()
         wordlists = create_wordlists(preferences.prefs["word_files"])
         print "Loading pattern files..."
-        patternfiles = constants.STANDARD_PATTERN_FILES + preferences.prefs["pattern_files"]
-        patterns = read_containers(patternfiles)
+        fs = constants.STANDARD_PATTERN_FILES + preferences.prefs[constants.PREF_PATTERN_FILES]
+        patterns = read_containers(fs)
         
         palabra = PalabraWindow()
-        palabra.wordlists.update(wordlists)
+        palabra.wordlists = wordlists
         palabra.patterns = patterns
         palabra.show_all()
         if has_splash:
