@@ -20,6 +20,8 @@ import operator
 import pangocairo
 
 import constants
+from editor import highlight_cells
+from grid import determine_scrabble_score
 
 class Histogram:
     def __init__(self, totals, width, height):
@@ -80,21 +82,36 @@ class Histogram:
         pcr.show_layout(layout)
         context.restore()
 
+def determine_words_message(puzzle, length=None):
+    """
+    Compute a sorted list of (x, y, d, word) of all words in the puzzle's grid.
+    If length is specified, return only words of that length.
+    """
+    words = []
+    for n, x, y, d, word, clue, explanation in puzzle.grid.gather_words():
+        if length is not None and len(word) != length:
+            continue
+        words.append((x, y, d, word.lower()))
+    words.sort(key=operator.itemgetter(3))
+    return words
+
 class PropertiesWindow(gtk.Dialog):
-    def __init__(self, palabra_window, puzzle):
-        gtk.Dialog.__init__(self, u"Puzzle properties", palabra_window
+    def __init__(self, window, puzzle):
+        gtk.Dialog.__init__(self, u"Puzzle properties", window
             , gtk.DIALOG_MODAL)
-        self.palabra_window = palabra_window
+        self.palabra_window = window
         self.puzzle = puzzle
-        self.connect("destroy", lambda widget: self.palabra_window.editor.clear_highlighted_words())
+        on_destroy = lambda widget: highlight_cells(window, puzzle, clear=True)
+        self.connect("destroy", on_destroy)
 
         status = puzzle.grid.determine_status(True)
-        message = self.determine_words_message(status, puzzle)
         
         tabs = gtk.Notebook()
+        tabs.set_property("tab-hborder", 8)
+        tabs.set_property("tab-vborder", 4)
         tabs.append_page(self.create_general_tab(status, puzzle), gtk.Label(u"General"))
         tabs.append_page(self.create_letters_tab(status, puzzle), gtk.Label(u"Letters"))
-        tabs.append_page(self.create_words_tab(message, status, puzzle), gtk.Label(u"Words"))
+        tabs.append_page(self.create_words_tab(status, puzzle), gtk.Label(u"Words"))
         tabs.append_page(self.create_metadata_tab(puzzle), gtk.Label(u"Metadata"))
         tabs.append_page(self.create_notepad_tab(puzzle), gtk.Label(u"Notepad"))
         tabs.connect("switch-page", self.on_switch_page)
@@ -111,13 +128,20 @@ class PropertiesWindow(gtk.Dialog):
         self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_ACCEPT)
         self.vbox.add(hbox)
         
+    def load_words(self, words):
+        self.words_data.clear()
+        for x, y, d, word in words:
+            txt = '<span font_desc="Monospace 12">' + word + '</span>'
+            self.words_data.append([txt, x, y, d])
+
     def on_switch_page(self, tabs, page, pagenum):
+        self.load_words(determine_words_message(self.puzzle))
         self.words_tab_sel.unselect_all()
-        self.palabra_window.editor.clear_highlighted_words()
+        highlight_cells(self.palabra_window, self.puzzle, clear=True)        
     
     def create_general_tab(self, status, puzzle):
         table = gtk.Table(13, 4, False)
-        table.set_col_spacings(18)
+        table.set_col_spacings(36)
         table.set_row_spacings(6)
         
         def create_header(table, title, x, y):
@@ -128,13 +152,19 @@ class PropertiesWindow(gtk.Dialog):
             label = gtk.Label("")
             label.set_alignment(1, 0)
             table.attach(label, x + 1, x + 2, y, y + 1)
-        def create_statistic(table, title, value, x, y):
+        def create_statistic(table, title, value, x, y, f=None):
             label = gtk.Label(title)
             label.set_alignment(0, 0)
             table.attach(label, x, x + 1, y, y + 1)
             label = gtk.Label(value)
             label.set_alignment(1, 0)
-            table.attach(label, x + 1, x + 2, y, y + 1)
+            if f is not None:
+                eventbox = gtk.EventBox()
+                eventbox.add(label)
+                eventbox.connect("button-press-event", f)
+                table.attach(eventbox, x + 1, x + 2, y, y + 1)
+            else:
+                table.attach(label, x + 1, x + 2, y, y + 1)
             
         create_header(table, u"<b>Grid</b>", 0, 0)
         create_statistic(table, u"Columns", str(puzzle.grid.width), 0, 1)
@@ -148,12 +178,18 @@ class PropertiesWindow(gtk.Dialog):
         create_statistic(table, u"Voids", str(status["void_count"]), 0, 6)
         create_statistic(table, u"Checked cells", str(status["checked_count"]), 0, 7)
         create_statistic(table, u"Unchecked cells", str(status["unchecked_count"]), 0, 8)
-        create_statistic(table, u"Open cells", str(status["open_count"]), 0, 9)
+        def on_open_click(widget, event):
+            highlight_cells(self.palabra_window, self.puzzle, f="open")
+        create_statistic(table, u"Open cells", str(status["open_count"]), 0, 9, on_open_click)
         
         create_header(table, u"<b>Words</b>", 2, 0)
-        create_statistic(table, u"Total words", str(status["word_count"]), 2, 1)
-        create_statistic(table, u"Across words", str(status["word_counts"]["across"]), 2, 2)
-        create_statistic(table, u"Down words", str(status["word_counts"]["down"]), 2, 3)
+        count_c = status["complete_count"]["across"] + status["complete_count"]["down"]
+        msg = ''.join([str(count_c), " / ", str(status["word_count"])])
+        create_statistic(table, u"Total words", msg, 2, 1)
+        msg = ''.join([str(status["complete_count"]["across"]), " / ", str(status["word_counts"]["across"])])
+        create_statistic(table, u"Across words", msg, 2, 2)
+        msg = ''.join([str(status["complete_count"]["down"]), " / ", str(status["word_counts"]["down"])])
+        create_statistic(table, u"Down words", msg, 2, 3)
         message = u"%.2f" % status["mean_word_length"]
         create_statistic(table, u"Average word length", message, 2, 4)
         msg = ''.join([str(status["clue_count"]), " / ", str(status["word_count"])])
@@ -161,7 +197,7 @@ class PropertiesWindow(gtk.Dialog):
         create_statistic(table, u"Connected?", "Yes" if status["connected"] else "No", 2, 6)
         
         create_header(table, u"<b>Score</b>", 2, 7)
-        score = self.determine_scrabble_score(puzzle)
+        score = determine_scrabble_score(puzzle.grid)
         create_statistic(table, u"Scrabble score", str(score), 2, 8)
         try:
             avg_score = float(score) / status["char_count"]
@@ -208,8 +244,7 @@ class PropertiesWindow(gtk.Dialog):
         self.histogram = Histogram(status["char_counts_total"], 312, 90)
         
         def on_char_click(widget, event, char):
-            self.palabra_window.editor.highlight_chars(char)
-        
+            highlight_cells(self.palabra_window, self.puzzle, "char", char)
         for y in xrange(0, 26, 6):
             for x, (char, count) in enumerate(status["char_counts_total"][y:y + 6]):
                 if count == 0:
@@ -290,7 +325,7 @@ class PropertiesWindow(gtk.Dialog):
         hbox.pack_start(main, True, True, 0)
         return hbox
         
-    def create_words_tab(self, message, status, puzzle):
+    def create_words_tab(self, status, puzzle):
         store = gtk.ListStore(int, int)
         tree = gtk.TreeView(store)
         self.words_tab_sel = tree.get_selection()
@@ -311,14 +346,19 @@ class PropertiesWindow(gtk.Dialog):
         window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         window.add(tree)
         
-        text = gtk.TextView()
-        text.set_editable(False)        
-        data = text.get_buffer()
-        data.set_text(message)
+        # word x y d
+        self.words_data = gtk.ListStore(str, int, int, str)
+        tree = gtk.TreeView(self.words_data)
+        cell = gtk.CellRendererText()
+        column = gtk.TreeViewColumn(u"Words", cell, markup=0)
+        tree.append_column(column)
+        tree.get_selection().connect("changed", self.on_word_selected)
+        
+        self.load_words(determine_words_message(self.puzzle))
         
         twindow = gtk.ScrolledWindow(None, None)
         twindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        twindow.add_with_viewport(text)
+        twindow.add(tree)
         
         label = gtk.Label()
         label.set_markup(u"<b>Words</b>")        
@@ -359,36 +399,27 @@ class PropertiesWindow(gtk.Dialog):
         store, it = selection.get_selected()
         if it:
             length = store.get_value(it, 0)
-            self.palabra_window.editor.highlight_words(length)
+            words = highlight_cells(self.palabra_window, self.puzzle, "length", length)
+            if not words:
+                words = determine_words_message(self.puzzle)
+            else:
+                words = determine_words_message(self.puzzle, length=length)
+            self.load_words(words)
             
-    @staticmethod
-    def determine_scrabble_score(puzzle):
-        # http://en.wikipedia.org/wiki/Scrabble_letter_distributions#English
-        scores = {'A': 1, 'B': 3, 'C': 3, 'D': 2, 'E': 1, 'F': 4, 'G': 2
-            , 'H': 4, 'I': 1, 'J': 8, 'K': 5, 'L': 1, 'M': 3, 'N': 1
-            , 'O': 1, 'P': 3, 'Q': 10, 'R': 1, 'S': 1, 'T': 1, 'U': 1
-            , 'V': 4, 'W': 4, 'X': 8, 'Y': 4, 'Z': 10, '': 0}
-        chars = [puzzle.grid.get_char(x, y) for x, y in puzzle.grid.cells()]
-        return sum([scores[c] for c in chars])
-        
-    @staticmethod
-    def determine_words_message(status, puzzle):
-        entries = puzzle.grid.entries()
-        entries.sort(key=len)
-        result = []
-        for word in entries:
-            result += (word + u"\n")
-        return ''.join(result[:-1])
+    def on_word_selected(self, selection):
+        store, it = selection.get_selected()
+        if it:
+            highlight_cells(self.palabra_window, self.puzzle, clear=True)
+            x, y, d = store[it][1], store[it][2], store[it][3]
+            highlight_cells(self.palabra_window, self.puzzle, "slot", (x, y, d))
         
     def create_metadata_tab(self, puzzle):
         details = gtk.Table(9, 2, False)
-        
         def metadata_changed(widget, key):
             value = widget.get_text().strip()
             self.puzzle.metadata[key] = value
             if len(value) == 0:
                 del self.puzzle.metadata[key]
-        
         def create_metadata_entry(key, label, index):
             entry = gtk.Entry(512)
             entry.connect("changed", lambda w: metadata_changed(w, key))
@@ -396,20 +427,18 @@ class PropertiesWindow(gtk.Dialog):
             alignment.add(gtk.Label(label))
             details.attach(alignment, 0, 1, index, index + 1)
             details.attach(entry, 1, 2, index, index + 1)
-            
             try:
                 value = puzzle.metadata[key]
                 entry.set_text(value)
             except KeyError:
                 pass
-            
-        create_metadata_entry("title", u"Title", 0)
-        create_metadata_entry("creator", u"Creator", 1)
-        create_metadata_entry("contributor", u"Editor", 2)
-        create_metadata_entry("rights", u"Rights", 3)
+        create_metadata_entry(constants.META_TITLE, u"Title", 0)
+        create_metadata_entry(constants.META_CREATOR, u"Author", 1)
+        create_metadata_entry(constants.META_EDITOR, u"Editor", 2)
+        create_metadata_entry(constants.META_COPYRIGHT, u"Rights", 3)
         #create_metadata_entry("description", u"Description", 4)
-        create_metadata_entry("publisher", u"Publisher", 4)
-        create_metadata_entry("date", u"Date", 5)
+        create_metadata_entry(constants.META_PUBLISHER, u"Publisher", 4)
+        create_metadata_entry(constants.META_DATE, u"Date", 5)
         #create_metadata_entry("identifier", u"Identifier", 7)
         #create_metadata_entry("language", u"Language", 8)
 
